require 'test/unit'

require 'simplezdb'
require 'date'

include SimpleZDB

class TC_DataBase < Test::Unit::TestCase

  def test_s_new
    db = DataBase.new
    assert_instance_of(DataBase, db)
    assert_nil(db.item_attrs)
  end

  def test_s_new2
    db = DataBase.new
    assert_raises(RuntimeError) { db.close }
  end

  def test_s_new3
    db = DataBase.new(DataBase::ADDRESSBOOK)
    assert_nothing_raised { db.close }
  end

  def test_s_open
    DataBase.open(DataBase::ADDRESSBOOK) {|db|
      assert_instance_of(DataBase, db)
    }
  end

  def test_size
    db = DataBase.new(DataBase::TODOLIST)
    assert_kind_of(Integer, db.size)
    assert(db.size > 0)
    db.close
  end

  def test_each
    db = DataBase.new(DataBase::MEMO)
    assert_nothing_raised {
      db.each {|card|
        assert_instance_of(Card, card)
        assert_equal(db, card.database)
        assert_kind_of(Integer, card.card_id)
        break
      }
    }
    db.close
  end

  def test_addressbook
    db = DataBase.new(DataBase::ADDRESSBOOK)
    assert_instance_of(DataBase, db)
    assert_nothing_raised { db.close }
  end

  def test_todolist
    db = DataBase.new(DataBase::TODOLIST)
    assert_instance_of(DataBase, db)
    assert_nothing_raised { db.close }
  end

  def test_datebook
    db = DataBase.new(DataBase::DATEBOOK)
    assert_instance_of(DataBase, db)
    assert_nothing_raised { db.close }
  end

  def test_mail_inbox
    db = DataBase.new(DataBase::MAIL_INBOX)
    assert_instance_of(DataBase, db)
    assert_nothing_raised { db.close }
  end

  def test_mail_outbox
    db = DataBase.new(DataBase::MAIL_OUTBOX)
    assert_instance_of(DataBase, db)
    assert_nothing_raised { db.close }
  end

  def test_memo
    db = DataBase.new(DataBase::MEMO)
    assert_instance_of(DataBase, db)
    assert_nothing_raised { db.close }
  end

  def test_item_attrs
    db = DataBase.new(DataBase::ADDRESSBOOK)
    item_attrs = db.item_attrs
    assert_instance_of(Array, item_attrs)
    assert(item_attrs.size > 0)
    assert_raises(NoMethodError) {
      db.item_attrs = "Something"
    }

    item_attrs.each do |attr|
      assert_instance_of(String, attr.name)
      assert_kind_of(Integer, attr.value_type)
    end
    db.close
  end

  def test_size
    db = DataBase.new(DataBase::MEMO)
    assert_kind_of(Integer, db.size)
    assert(db.size >= 0)
    db.close
  end

  def test_read_item
    db = DataBase.new(DataBase::ADDRESSBOOK)
    item_attrs = db.item_attrs
    db.each do |card|
      attr = item_attrs[rand(item_attrs.size)]
      item = db.read_item(attr.name, card.card_id)
      assert_not_nil(item)
      break
    end
    db.close
  end

  def test_read_time_field
    db = DataBase.new(DataBase::DATEBOOK)
    item_attrs = db.item_attrs
    time_field = nil
    item_attrs.each_with_index do |attr, i|
      if attr.value_type == ItemAttr::TYPE_TIME then
        time_field = i
        break
      end
    end
    assert_not_nil(time_field)
    db.each do |card|
      item = db.read_time_field(time_field, card.card_id)
      assert_kind_of(Date, item)
      break
    end
    db.close
  end

  def test_read_categories
    db = DataBase.new(DataBase::ADDRESSBOOK)
    db.each do |card|
      categories = db.read_categories(card.card_id)
      assert_instance_of(Array, categories)
      if categories.size > 0 then
        assert_instance_of(String, categories.first)
        break
      end
    end
    db.close
  end
end

class TC_Card < Test::Unit::TestCase
  def setup
    @db = DataBase.new(DataBase::DATEBOOK)
    @card = nil
    @db.each do |card|
      @card = card
      break
    end
  end

  def teardown
    @db.close
  end

  def test_card_id
    assert_kind_of(Integer, @card.card_id)
  end

  def test_database
    assert_instance_of(DataBase, @card.database)
  end

  def test_keys
    assert_instance_of(Array, @card.keys)
  end

  def test_aref
    key = @card.keys[rand(@card.keys.size)]
    assert_not_nil(@card[key])
  end

  def test_aset
    str_key = nil
    @card.database.item_attrs.each do |attr|
      if attr.value_type == ItemAttr::TYPE_TEXT then
        str_key = attr.name
        break
      end
    end
    assert_not_nil(str_key)
    orig = @card[str_key]
    @card[str_key] = 'Foo'
    assert_equal(@card[str_key], 'Foo')
    @card[str_key] = orig
    assert_equal(@card[str_key], orig)
  end

  def test_each
    size = @card.database.item_attrs.size
    count = 0
    @card.each do |val|
      count += 1
    end
    assert_equal(size, count)
  end

  def test_to_ary
    assert_instance_of(Array, @card.to_a)
  end

  def test_to_hash
    assert_instance_of(Hash, @card.to_hash)
  end

  def test_categories
    assert_instance_of(Array, @card.categories)
    if @card.categories.size > 0 then
      assert_instance_of(String, @card.categories.first)
    end
  end

  def test_categories_assign
    @card.categories = ["Home"]
    assert_equal(["Home"], @card.categories)
  end
end

class TC_CategorySet < Test::Unit::TestCase
  def setup
    @cs = CategorySet.new
  end

  def test_to_a
    categories = @cs.to_a
    assert_kind_of(Array, categories)
    if categories.size > 0 then
      assert_kind_of(String, categories.first)
    end
  end

  def test_include_p
    ["Friend", "God"].each do |label|
      if @cs.include?(label) then
        id = @cs.id_of(label)
        assert_equal(label, @cs[id])
      else
        assert_nil(@cs.id_of(label))
      end
    end
  end
end
